//
// DXGL.cpp
//
// Written by Adam Dawes (www.adamdawes.com)
// 17th May, 2002
//
// May be distributed freely.
//

#include "DXGL.h"


// Matrix stack for DX operations
#define DXSTACKSIZE 40
D3DXMATRIX mDXMatrixStack[DXSTACKSIZE];
int mDXStackPointer=0;



// Apply the current matrix to the Direct3D device
void dxApplyMatrix(IDirect3DDevice8 *D3DDevice)
{
	// Set the transformation matrix for the device
	D3DDevice->SetTransform(D3DTS_WORLD, &mDXMatrixStack[mDXStackPointer]);
}

// Load the identity matrix
void dxLoadIdentity(void)
{
	// Load an identity matrix to the current matrix stack element
	D3DXMatrixIdentity(&mDXMatrixStack[mDXStackPointer]);
}

// Translate the current matrix
void dxTranslatef(float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Multiply the current matrix by the required translation matrix
	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixTranslation(&matrixTemp, fX, fY, fZ), &mDXMatrixStack[mDXStackPointer]);
}

// Rotate the current matrix (in degrees)
void dxRotatef(float fAngle, float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Vector representing the axis of rotation
	D3DXVECTOR3 rotMatrix(fX, fY, fZ);

	// Scale the angle from degrees to radians
	fAngle *= DEG2RAD;

	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixRotationAxis(&matrixTemp, &rotMatrix, fAngle), &mDXMatrixStack[mDXStackPointer]);
}

// Scale the current matrix
void dxScalef(float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Scale the current matrix
	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixScaling(&matrixTemp, fX, fY, fZ), &mDXMatrixStack[mDXStackPointer]);
}

// Push the current matrix onto the stack
bool dxPushMatrix(void)
{
	// Is there any room left on the stack?
	if (mDXStackPointer < DXSTACKSIZE-1)
	{
		// Yes, so increase the stack pointer
		mDXStackPointer += 1;

		// Copy the previous matrix to this stack position
		mDXMatrixStack[mDXStackPointer] = mDXMatrixStack[mDXStackPointer-1];

		// All ok
		return true;
	}

	// No space on stack
	return false;
}

// Pop the stack
bool dxPopMatrix(void)
{
	// Is there anything remaining on the stack?
	if (mDXStackPointer > 0)
	{
		// Yes, so decrease the stack pointer
		mDXStackPointer -= 1;

		// All ok
		return true;
	}

	// Nothing left on stack
	return false;
}